{% load i18n %} 
{% autoescape off %} 
{% blocktrans %} 
You're receiving this
email because you requested a password reset for your user account at {{ site_name }}.

Please go to the following page and choose a new password: 
{% endblocktrans %} 
{{ protocol }}://{{ domain }}{% url 'django_accounts_api:password_reset_confirm' uidb64=uid token=token %} 
{%blocktrans %} 

Your username, in case you've forgotten: {{ user.get_username }}.
{% endblocktrans %} 
{% blocktrans %} 

Thanks for using our site! The 
{{ site_name}} team 

{% endblocktrans %} 
{% endautoescape %}
